/*
 * npc-key
 * $Id: $
 */

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
//#include <limits.h>
#include <string.h>
#include <unistd.h>
#include "rand_gen.h"

#define MAX_SIZE 2147483647

void usage(char *prog)
{
  fprintf(stderr,"USAGE: %s [opts] -s output_size -k key_filename\n\n",prog);
  fprintf(stderr,"  opts are v h ?\n");
  fprintf(stderr,"    -v         verbose output\n");
  fprintf(stderr,"    -h or -?   this help screen\n");
  fprintf(stderr,"\n");
}

int main(int argc, char** argv)
{
  char *prog;
  FILE* fd = NULL;
  char * kfile = NULL;
  unsigned char randMaterial[256];
  unsigned long kfilesize = 0, pfilesize = 0;
  char ch;
  char *t;
  int i = 0, j = 0, verbose = 0;

  prog = argv[0];

  while((ch = getopt(argc, argv, "?hvs:k:")) != EOF)
  {
    switch(ch)
    {
      case '?':
      case 'h':
        usage(prog);
        exit(0);
      case 'v':
        verbose = 1;
        break;
      case 'k':
        kfile = (char *)strdup(optarg);
        fd = fopen(kfile,"wb");
        if(fd == NULL)
        {
          fprintf(stderr,"=== Cannot open file for writing: %s\n",kfile);
          exit(-1);
        }
        break;
      case 's':
        kfilesize = strtoul(optarg,&t,10);
        if(*t == 0)
        {
          if(kfilesize > MAX_SIZE)
          {
            fprintf(stderr,"=== Request file size %s is larger than %d\n",optarg,MAX_SIZE);
            exit(-1);
          }
        }
        else
        {
          fprintf(stderr,"=== Unable to convert requested file size: %s\n",optarg);
          exit(-1);
        }
        break;
      default:
        usage(prog);
        exit(-1);
    }
  }
  /* post arg processing */
  if(kfile == NULL)
  {
    fprintf(stderr,"=== You must specify a filename for key output\n");
    usage(prog);
    exit(-1);
  }

  if(kfilesize < 1)
  {
    fprintf(stderr,"=== You must specify a filesize for key output\n");
    usage(prog);
    exit(-1);
  }
  /* end post arg processing */

  if(seed_rand_num() == -1)
  {
    fprintf(stderr,"=== Unable to seed PRNG\n");
    exit(-1);
  }

  while(pfilesize < kfilesize)
  {
    gen_rand_mat(&randMaterial[0],256);
    if((kfilesize - pfilesize) < 256)
    {
      j = kfilesize - pfilesize;
      if(fwrite(randMaterial,j,1,fd) != 1)
      {
        fprintf(stderr,"=== Error writing to key file: %s\n",kfile);
        fprintf(stderr,"=== Wrote %d bytes of requested %d\n",(int)pfilesize,(int)kfilesize);
        fclose(fd);
        exit(-1);
      }
    }
    else
    {
      if(fwrite(randMaterial,256,1,fd) != 1)
      {
        fprintf(stderr,"=== Error writing to key file: %s\n",kfile);
        fprintf(stderr,"=== Wrote %d bytes of requested %d\n",(int)pfilesize,(int)kfilesize);
        fclose(fd);
        exit(-1);
      }
    }
    pfilesize += 256;
    
    if(verbose)
    {
      if(i == 999)
      {
        fprintf(stdout,"=== Working %d\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b",(int)pfilesize);
        i = 0;
      }
      i++;
    }
  }
  if(verbose) fprintf(stdout,"=== Complete %d                         \n",(int)pfilesize);
  fclose(fd);
  exit(0);
} 
